Let's say that you have a [Sinatra](http://sinatrarb.com/) web application allowing users to kick off a computationally-intensive command. Perhaps it's a build system. Perhaps it's a simulated annealing solution to solving sports scheduling. If you do something like this:

    get '/start' do
      @result = do_long_running_thing()
      haml :results
    end

…then your users will visit the URL and wait minutes before anything happens. Very likely their web browser will timeout and close the connection. Even worse, other visitors to your website will be stalled, waiting for the single-threaded Ruby process that is your web site to finish what it was working on. This is clearly not acceptable.

You could sort of fix the problem of locking out other users by [running many website processes behind a reverse proxy](/nginx-as-reverse-proxy-cache-for-thin), but that's not a scalable solution: are you really going to have one ruby process for each possible concurrent visitor? Further, it still doesn't provide a good experience for the user of the page.

We could kick off a [Thread](http://www.ruby-doc.org/stdlib-1.9.3/libdoc/thread/rdoc/index.html) in the web server process, but something about this feels dirty. Instead, here's our plan:

1.  We want the visitor kicking off the long-running command to immediately see a page letting them know that it's running. This provides good feedback, and frees up the server to handle other requests.

2. We'll run the long-running command in a completely different process. An entirely different Ruby program. We'll run each new command in its own thread.

3. We'll provide a way for our Sinatra web application to poll the other process and see how things are going. The web page will make periodic AJAX requests to the server, the server will ask the other process for an update and will respond to the AJAX request with some JSON, and the web page will update the progress.

----

You can start the DRb server and web server with:

    ruby mcp.rb &
    ruby webserver.rb

You don't have to start the MCP before you start the webserver. The line of code:

    MCP = DRbObject.new_with_uri(DRBSERVER)

tells the web server _how_ to connect to the DRb server when it needs to; it does not attempt to connect immediately. You can even restart the DRb server if necessary, and your web server will reconnect at the right time.

We use the short name `MCP` in the web server not only a geeky reference to Tron, but also to illustrate the fact that although this object acts _just like_ the `MasterControlProcess` module running on the other end of the DRb connection, it is _not_ the same object. We do not need to include the `mcp.rb` file in our web server; the DRb connection sends messages over the wire and the actual `MasterControlProcess` on the other end handles them and passes the response back over the wire.

We use a `Thread` to do the bulk of work in the `Process` so that the DRb server remains responsive. We use a `Mutex` to ensure that when the MasterControlProcess is reading the information it is not being changed at the same time by the process. We don't wrap reads the the `results` in a `Mutex` because we assume that the thread will have completed its work—and written to the results—by the time anyone gets around to asking for the results.
